@@ -831,6 +831,14 @@ if (this instanceof Window && this.document) {
 		Evo.initializeAndPostContentLoaded(this.document);
 }
 
+Evo.replaceImgSource = function(src, requireFirst, first, second)
+{
+	if (requireFirst)
+		return src.replace(second, first);
+
+	return src.replace(first, second);
+}
+
 Evo.vCardCollapseContactList = function(elem)
 {
 	if (elem && elem.id && elem.id != "" && elem.ownerDocument) {
@@ -844,7 +852,8 @@ Evo.vCardCollapseContactList = function(elem)
 
 			for (child = elem.firstElementChild; child; child = child.nextElementSibling) {
 				if (/*child instanceof HTMLImageElement*/ child.tagName == "IMG") {
-					child.src = list.hidden ? "gtk-stock://pan-end-symbolic" : "gtk-stock://pan-down-symbolic";
+					child.src = Evo.replaceImgSource(child.src, list.hidden,
+						"gtk-stock://x-evolution-pan-end", "gtk-stock://x-evolution-pan-down");
 				}
 			}
 		}
@@ -967,12 +976,15 @@ Evo.mailDisplayResizeContentToPreviewWidthCB = function(timeStamp)
 									expected_width = parseInt(tmp.slice(0, -2));
 								else
 									expected_width = tmp;
-								img.setAttribute("x-evo-width-modified", expected_width);
+								if (expected_width > 0)
+									img.setAttribute("x-evo-width-modified", expected_width);
+							}
+							if (expected_width > 0) {
+								if (expected_width < local_width)
+									img.style.width = expected_width + "px";
+								else
+									img.style.width = local_width + "px";
 							}
-							if (expected_width < local_width)
-								img.style.width = expected_width + "px";
-							else
-								img.style.width = local_width + "px";
 						}
 					}
 				}
@@ -1057,8 +1069,12 @@ Evo.mailDisplayToggleHeadersVisibility = function(elem)
 	full_headers.style.setProperty("display", expanded ? "none" : "table");
 	short_headers.style.setProperty("display", expanded ? "table" : "none");
 
-	if (elem.firstElementChild && /* elem.firstElementChild instanceof HTMLImageElement */ elem.firstElementChild.tagName == "IMG") {
-		elem.firstElementChild.src = expanded ? "gtk-stock://pan-end-symbolic" : "gtk-stock://pan-down-symbolic";
+	var child;
+
+	for (child = elem.firstElementChild; child; child = child.nextElementSibling) {
+		if (/*child instanceof HTMLImageElement*/ child.tagName == "IMG")
+			child.src = Evo.replaceImgSource(child.src, expanded,
+				"gtk-stock://x-evolution-pan-end", "gtk-stock://x-evolution-pan-down");
 	}
 
 	window.webkit.messageHandlers.mailDisplayHeadersCollapsed.postMessage(expanded);
@@ -1095,7 +1111,8 @@ Evo.mailDisplayToggleAddressVisibility = function(elem)
 
 		full_addr.style.setProperty("display", expanded ? "none" : "inline");
 		ellipsis.style.setProperty("display", expanded ? "inline" : "none");
-		img.src = expanded ? "gtk-stock://pan-end-symbolic" : "gtk-stock://pan-down-symbolic";
+		img.src = Evo.replaceImgSource(img.src, expanded,
+			"gtk-stock://x-evolution-pan-end", "gtk-stock://x-evolution-pan-down");
 	}
 }
 
@@ -1482,6 +1499,28 @@ Evo.MailDisplayProcessMagicSpacebar = function(towards_bottom)
 	Evo.mailDisplayUpdateMagicSpacebarState();
 }
 
+Evo.MailDisplayManageInsecureParts = function(iframe_id, part_id_prefix, show, partids)
+{
+	var ii, elem;
+
+	elem = Evo.FindElement(iframe_id, "show:" + part_id_prefix);
+	if (elem) {
+		elem.hidden = show
+	}
+
+	elem = Evo.FindElement(iframe_id, "hide:" + part_id_prefix);
+	if (elem) {
+		elem.hidden = !show
+	}
+
+	for (ii = 0; ii < partids.length; ii++) {
+		elem = Evo.FindElement(iframe_id, partids[ii]);
+
+		if (elem)
+			elem.hidden = !show;
+	}
+}
+
 var EvoItip = {
 	SELECT_ESOURCE : "select_esource",
 	TEXTAREA_RSVP_COMMENT : "textarea_rsvp_comment",
@@ -1852,3 +1891,30 @@ EvoItip.GetState = function(iframe_id)
 
 	return res;
 }
+
+EvoItip.FlipAlternativeHTMLPart = function(iframe_id, element_value, img_id, span_id)
+{
+	var elem = Evo.FindElement(iframe_id, element_value);
+	if (elem) {
+		elem.hidden = !elem.hidden;
+	}
+	elem = Evo.FindElement(iframe_id, img_id);
+	if (elem) {
+		var tmp = elem.src;
+		elem.src = elem.getAttribute("othersrc");
+		elem.setAttribute("othersrc", tmp);
+	}
+	elem = Evo.FindElement(iframe_id, img_id + "-dark");
+	if (elem) {
+		var tmp = elem.src;
+		elem.src = elem.getAttribute("othersrc");
+		elem.setAttribute("othersrc", tmp);
+	}
+	elem = Evo.FindElement(iframe_id, span_id);
+	if (elem) {
+		var tmp = elem.innerText;
+		elem.innerText = elem.getAttribute("othertext");
+		elem.setAttribute("othertext", tmp);
+	}
+	window.webkit.messageHandlers.scheduleIFramesHeightUpdate.postMessage(0);
+}
